#!/bin/sh
# $Id: smtp2.sh,v 1.45 2005/03/30 20:00:39 ca Exp $
# Copyright (c) 2003-2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
#
# test program running the full system:
# qmgr, smar, smtpc, smtps
# using smtps and smtpc from statethreads/examples as source/sink
# this program tests some failure modes, e.g., temporary/permanent
# errors from the sink.
#
# This is a fairly versatile test program that can be used by other
# scripts. However, smtp1.sh is probably the better choice.
#
test -s stop && exit 1
#
# source directory of this (and other) shell script
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
#
EDBOUT=edb.out
EDBROUT=edbr.out
EDBOK=edbr.ok
# default settings
TAS=1
THREADS=1
RCPTS=1
TOTAL=1
TR=2
INIT=true
RMDEFEDB=true
RCPTADDR=""
SENDER="a@b.c"
# initial delay for retries by QMGR
DELAY_TRY=600
# after this amount of time the SMTP sink will accept all mails
OKDELAY=""
# other options to pass to QMGR
QOPTS=""
# debug flags
D="-d 5"
# time to sleep after running test before terminating programs
SLEEP=1
# server options
SRVOPTS=${SM_SERVER_OPTIONS}

# works only with ksh on OSF/1
if test X"`uname`" != "XOSF1"
then
while getopts d:Di:Mn:o:R:s:S:T:t:Q:V: FLAG
do
  case "${FLAG}" in
    d)
       D="-d ${OPTARG}"
       ;;
    D) INIT=false;;
    M) RMDEFEDB=false;;
    i)
       DELAY_TRY="${OPTARG}"
       ;;
    o)
       OKDELAY="-o ${OPTARG}"
       ;;
    n)
       # RCPTS="${OPTARG}"
       ;;
    R)
       RCPTADDR="${RCPTADDR} -R${OPTARG}"
       ;;
    s)
       SLEEP="${OPTARG}"
       ;;
    S)
       SENDER="${OPTARG}"
       ;;
    T)
       # TAS="${OPTARG}"
       ;;
    t)
       # THREADS="${OPTARG}"
       ;;
    Q)
       QOPTS="${OPTARG}"
       ;;
    V)
       SRVOPTS="${OPTARG}"
       ;;
    ?)
       echo "$0: unknown option ${FLAG}"
       echo "$0: options:"
       echo "-d n	debug level"
       echo "-D n	do not initialize test environment"
       echo "-i n	initial delay for retries in QMGR scheduler"
       echo "-o n	after n seconds accept all mails in SMTP sink"
       echo "-R address	recipient address"
       echo "-s n	sleep n seconds before terminating system"
       echo "-S address	sender address"
       #echo "-n n	n recipients per transaction"
       #echo "-T n	n transactions"
       #echo "-t n	n threads"
       echo "-Q opts	options for QMGR"
       echo "-V opts	additional SMTP server options"
       exit 1
       ;;
  esac
done
shift `expr ${OPTIND} - 1`
TOTAL=`expr ${TAS} \* ${THREADS}`
TR=`expr ${RCPTS} \* ${TOTAL} \* 2`
fi
#
# setup/initialize system
. ${SD}/mta-setup.sh
#

killit()
{
if test -s ${PIDS}
then
  # stop MTA components
  for i in `cat ${PIDS}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall qmgr smar
fi
if test X"${SRVPID}" != "X"
then
  kill -TERM ${SRVPID}
fi
exit 1
}

SRVPID=""
# store the PIDs for the started programs
rm -f ${PIDS}
trap killit 2

# Note: the test programs which rely on failures from SMAR work only if
# SMAR has been compiled with -DSMAR_TEST!
SMAR_TEST=false
../smar/smar -D | ${GREP} "SMAR_TEST" >/dev/null && SMAR_TEST=true

# start SMTP sink
${STE}/smtps2 ${PIPELINING} ${OKDELAY} -i -l . -b ${H}:${SNKPORT} > ${SRVL} 2>&1 &
status=$?
SRVPID=$!
if test "x${status}" != "x0"
then
  echo "FAIL: cannot start server ${status}"
  exit ${status}
fi
sleep 1

# version of output files
V=0
# path to programs
P=..
# file to override settings
# test -f options && . ./options
# go for it
${P}/smar/smar ${D} $@ > a${V}.log 2>&1 &
echo $! >> ${PIDS}
#${CHKD}/qmgr-running.sh "${SMARSOCK}" || killit
${P}/qmgr/qmgr -w ${W4S} -W ${W4C} ${QOPTS} ${D} -D ${DELAY_TRY} $@ > q${V}.log 2>&1 &
echo $! >> ${PIDS}
# wait a bit for startup such that the sockets are created
#export SMTPCSOCK
#${CHKD}/qmgr-running.sh || killit

${P}/smtpc/smtpc -w ${W4S} -l . -P ${SNKPORT} ${D} $@ > c${V}.log 2>&1 &
echo $! >> ${PIDS}
${P}/smtps/smtps -w ${W4S} ${SRVOPTS} -l . ${D} -b localhost:${SRVPORT} $@ > s${V}.log 2>&1 &
echo $! >> ${PIDS}

# check that the MTA is running
export SRVPORT P
${CHKD}/mta-running.sh || killit

if test X"${RCPTADDR}" != X
then
  if ${STE}/smtpc -r ${H}:${SRVPORT} -f${SENDER} ${RCPTADDR} -n "${RCPTS}" -T "${TAS}" -t "${THREADS}" > ${CLTL} 2>&1
  then
    :
  else
    cat ${CLTL}
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  # send messages
  # 1. temp fail recipient <4xy@...>
  # 2. perm fail recipient <5xy@...> with sender that cause temp AR error
  if ${STE}/smtpc -r ${H}:${SRVPORT} -fa@b.c -R451@x.z -n "${RCPTS}" -T "${TAS}" -t "${THREADS}" > ${CLTL} 2>&1
  then
    :
  else
    cat ${CLTL}
    ERRS=`expr ${ERRS} + 1 `
  fi
  if test X"${SMAR_TEST}" = "Xfalse"
  then
    echo "test program will fail because smar is not compiled with -DSMAR_TEST"
  fi
  if ${STE}/smtpc -r ${H}:${SRVPORT} -fa2@t.z -R550@x.z -n "${RCPTS}" -T "${TAS}" -t "${THREADS}" > ${CLTL} 2>&1
  then
    :
  else
    cat ${CLTL}
    ERRS=`expr ${ERRS} + 1 `
  fi
fi

sleep ${SLEEP}

# stop MTA components
for i in `cat pids`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall qmgr smar
fi

# send SMTP server a signal to dump its state
if kill -USR1 ${SRVPID}
then
  :
else
  echo "kill -USR1 failed: $?"
  exit 1
fi
sleep 2

# check state of SMTP server
if ${GREP} "^Address *${H}:${P}" ${SRVL} > /dev/null
then
  :
else
  echo "failed to listen on ${H}:${P}"
  ERRS=`expr ${ERRS} + 1 `
fi
if ${GREP} "^Thread.* 8/" ${SRVL} > /dev/null
then
  :
else
  echo "failed to start threads"
  ERRS=`expr ${ERRS} + 1 `
fi
# the number of requests can vary based on the scheduler...
RQS=`${GREP} "^Requests served *" ${SRVL} | sed -e 's/^Requests served *//'`
if test "${RQS}" -lt "${TOTAL}"
then
  echo "wrong number of requests: expected=${TOTAL} got=${RQS}"
  ERRS=`expr ${ERRS} + 1 `
fi
if ${GREP} "^Recipients *${TR}$" ${SRVL} > /dev/null
then
  :
else
  echo "wrong number of recipients: expected=${TR}"
  ${GREP} "^Recipients" ${SRVL}
  ERRS=`expr ${ERRS} + 1 `
fi
if ${GREP} "total=${TOTAL} " ${CLTL} > /dev/null
then
  :
else
  echo "wrong number of mails sent: expected=${TOTAL}"
  ${GREP} "total=" ${CLTL}
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------
kill -TERM ${SRVPID}

# ----------------
if ${IDBR0} >/dev/null
then
  if test "${ERRS}" = "0"
  then
    rm -f ibdd/ibd00000000*
  fi
else
  echo "t-idbr-0 failed"
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------
cat > ${EDBOK} <<EOF
got transaction
	mail=<a@b.c>
	rcpts_tot=1
	rcpts_left=1
	rcpts_temp=1
	rcpts_perm=0
	state=451
	aqt_rcpts_ar=0
got recipient
	rcpt=<451@x.z>
	aqr_rcpt_idx=0
	aqr_da_idx=0
	aqr_status=451
got transaction
	mail=<a2@t.z>
	rcpts_tot=2
	rcpts_left=1
	rcpts_temp=1
	rcpts_perm=0
	state=481
	aqt_rcpts_ar=0
got recipient
	rcpt=<a2@t.z>
	aqr_rcpt_idx=1
	aqr_da_idx=0
	aqr_status=481
EOF

if ${EDBR} > ${EDBROUT} 2>&1
then
  egrep -v 'tests completed successfully|time|(id|cdb|srv_ip4)=' ${EDBROUT} > ${EDBOUT} 2>&1
  if diff ${EDBOUT} ${EDBOK}
  then
    :
  else
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  echo "${EDBR} failed"
  ERRS=`expr ${ERRS} + 1 `
fi

# ----------------
for i in a c q s
do
  if egrep 'ERROR|CRIT|ALERT' ${i}${V}.log
  then
    ERRS=`expr ${ERRS} + 1 `
  fi
done

# ----------------
if test "${ERRS}" = "0"
then
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
