#!/bin/sh
# $Id: t-mts-ocr-2.sh,v 1.7 2005/04/21 21:30:59 ca Exp $
# Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test outgoing connection restrictions.
# two tests: concurrency limit 1/2, check number of busy threads in sink;
# all mails should be delivered within a "short" timeframe, i.e.,
# the scheduler should be activated whenever a session close to keep
# a "steady" delivery rate (make maximum use of allowed limits).
# uses mcpmts.sh
# ----------------------------------------
#
test -s stop && exit 1
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh
# currently set in mta-setup.sh
CLTL=sink.log

I=""
OUT="t-mts-ocr-2.out"

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create directory to pass socket
if test -d smtps
then
  :
else
  mkdir smtps
fi

killit()
{
rm -f conf.db mt
exit 1
}

rm -f conf.db mt ${OUT}

# create restrictions: maximum concurrency: 1
cat > conf <<EOF
oci:10.1.1.2	1
ocm:10.1.1.2	1
EOF
# create map
${MM} -t'	' -F conf.db < conf
trap killit 2

# ----------------------------------------
cat >${SMXCNF} <<EOF
smtps { log_level = 14; pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	port = ${SRVPORT}; mcp_type = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -f ${SMXCNF} ${SM_SERVER_OPTIONS} -d 3";
}
smtpc { log_level = 14; remote_port = ${SNKPORT}; wait_for_server=4;
	connect_only_to = 127.0.0.1;
	io_timeout=60s;
	mcp_type = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF} -d 3";
}
qmgr { log_level = 14; wait_for_server=4; wait_for_client=4; 
	retry_min_delay = 300;
	mcp_type = wait; user = ${USER};
	# restartdependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF} -d 3";
}
smar { log_level = 14;
	mcp_type = wait; user = ${USER};
	# restartdependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

cat > mt <<EOF
dom1.tld	[10.1.1.1]
dom2.tld	[10.1.1.2]
dom3.tld	[10.1.1.3]
dom4.tld	[10.1.1.4]
dom5.tld	[10.1.1.5]
dom6.tld	[10.1.1.6]
dom7.tld	[10.1.1.7]
dom8.tld	[10.1.1.8]
dom9.tld	[10.1.1.9]
EOF

# ----------------------------------------
# 
TEST=1
if echo "${SKIP} X" | ${GREP} -v "${TEST} " > /dev/null
then

for i in smar smtps smtpc qmgr
do
  cp /dev/null ${i}.log
done

if ${SHELL} ${SD}/mcpmts.sh -Y '-w1' -c '-s 10 -D dom2.tld -F dom1.tld' -N -s 20 -A 10 -E 10
then
  if ${GREP} 'Max busy threads .* 1$' ${CLTL} >/dev/null
  then
    ${VERBOSE} && echo "test ${TEST} succeeded"
  else
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "test ${TEST} failed"
  test X"${STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
rm -f conf.db ${OUT}
# create restrictions: maximum concurrency: 2
cat > conf <<EOF
oci:10.1.1.2	2
ocm:10.1.1.2	2
EOF
# create map
${MM} -t'	' -F conf.db < conf
trap killit 2

# ----------------------------------------
# 
TEST=2
if echo "${SKIP} X" | ${GREP} -v "${TEST} " > /dev/null
then

for i in smar smtps smtpc qmgr
do
  cp /dev/null ${i}.log
done

if ${SHELL} ${SD}/mcpmts.sh -Y '-w1' -c '-s 10 -D dom2.tld -F dom1.tld' -N -s 20 -A 10 -E 10
then
  if ${GREP} 'Max busy threads .* 2$' ${CLTL} >/dev/null
  then
    ${VERBOSE} && echo "test ${TEST} succeeded"
  else
    ERRS=`expr ${ERRS} + 1 `
  fi
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "test ${TEST} failed"
  test X"${STOPONERROR}" != X && exit 1
fi
fi



# -----------------
rm -f conf.db mt ${OUT}

# ----------------------------------------
# end
if test "${ERRS}" = "0"
then
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
