/*
 * Copyright (c) 2002-2004 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: pmilter.h,v 1.11 2005/03/21 22:57:39 ca Exp $
 */

#ifndef PMILTER_H
#define PMILTER_H	1

#include "sm/generic.h"
#include "sm/magic.h"
#include "sm/pthread.h"
#include "sm/heap.h"
#include "sm/net.h"
#include "sm/io.h"
#include "sm/evthr.h"
#include "sm/rcb.h"
#include "sm/rcbl.h"
#include "sm/rcbcomm.h"
#include "sm/mta.h"
#include "sm/queue.h"

#ifndef SM_USE_PMILTER
#define SM_USE_PMILTER	1
#endif

#ifndef PMILTER_DEBUG
#define PMILTER_DEBUG	1
#endif

#if SM_HEAP_CHECK
extern SM_DEBUG_T SmHeapCheck;
# define HEAP_CHECK (SmHeapCheck > 0)
#else
# define HEAP_CHECK 0
#endif

typedef struct pmilt_ctx_S	pmilt_ctx_T, *pmilt_ctx_P;
typedef struct pmss_ctx_S	pmss_ctx_T, *pmss_ctx_P;
typedef struct pmse_ctx_S	pmse_ctx_T, *pmse_ctx_P;

/* HACK .... */
#define MAX_SMTPS_FD	32	/* must <= 32, see pmilt_csused */

#define SM_PMILT_CTX_MAGIC	SM_MAGIC('P', 'M', 'I', 'C')
#define SM_PMSS_CTX_MAGIC	SM_MAGIC('P', 'M', 'S', 'S')
#define SM_PMSE_CTX_MAGIC	SM_MAGIC('P', 'M', 'S', 'E')
#define SM_IS_PMILT_CTX(qmgr_ctx) SM_REQUIRE_ISA((pmilt_ctx), SM_PMILT_CTX_MAGIC)
#define SM_IS_PMSS_CTX(qmgr_ctx) SM_REQUIRE_ISA((pmss_ctx), SM_PMSS_CTX_MAGIC)
#define SM_IS_PMSE_CTX(qmgr_ctx) SM_REQUIRE_ISA((pmss_ctx), SM_PMSE_CTX_MAGIC)

/*
**  pmilter context
*/

struct pmilt_ctx_S
{
	sm_magic_T	 sm_magic;
	pthread_mutex_t	 pmilt_mutex;

	char *		 pmilt_sockname;

	int		 pmilt_sslfd;	/* listen fd */
	unsigned int	 pmilt_ssnfd;	/* number of used fds */
	uint32_t	 pmilt_sused;	/* bitmask for used elements */
	pmss_ctx_P	 pmilt_sctx[MAX_SMTPS_FD];

	sm_evthr_ctx_P	 pmilt_ev_ctx;	/* event thread context */

};

/*
**  Session context in pmilter
**  These are currently organized in a list; a hash table is more efficient
**  if there are many concurrent sessions. ENHANCE
**  Note: we don't need to keep an entire SMTP session context around,
**  this is just for pmilter calls which can take only a few arguments.
*/

struct pmse_ctx_S
{
	sm_magic_T	 sm_magic;
	pmilt_ctx_P	 pmse_pmilt_ctx;	/* pointer back to main ctx */
	void		*pmse_appl_ctx;	/* application ctx */
	smtp_id_T	 pmse_se_id;	/* SMTPS session id */
	smtp_id_T	 pmse_ta_id;	/* SMTPS transaction id */
	sm_str_P	 pmse_arg1;
	sm_str_P	 pmse_arg2;
	rcpt_idx_T	 pmse_rcpt_idx;
	TAILQ_ENTRY(pmse_ctx_S)	 pmse_link;	/* link to next elem */
};

#define PMSEL_INIT(pmss_ss_hd)	TAILQ_INIT(pmss_ss_hd)
#define PMSEL_FIRST(pmss_ss_hd)	TAILQ_FIRST(pmss_ss_hd)
#define PMSEL_END(pmss_ss_hd)	TAILQ_END(pmss_ss_hd)
#define PMSEL_EMPTY(pmss_ss_hd)	TAILQ_EMPTY(pmss_ss_hd)
#define PMSEL_NEXT(pmse_ctx)		TAILQ_NEXT(pmse_ctx, pmse_link)
#define PMSEL_PRE(pmss_ss_hd, pmse_ctx)	TAILQ_INSERT_HEAD(pmss_ss_hd, pmse_ctx, pmse_link)
#define PMSEL_APP(pmss_ss_hd, pmse_ctx)	TAILQ_INSERT_TAIL(pmss_ss_hd, pmse_ctx, pmse_link)
#define PMSEL_REMOVE(pmss_ss_hd, pmse_ctx) TAILQ_REMOVE(pmss_ss_hd, pmse_ctx, pmse_link)


/*
**  Task context pmilter/SMTP server
**  Generally access to this structure is restricted since it is associated
**  with exactly one fd and hence one (evthr) task.
**  However, see below for details which parts are protected by mutexes.
*/

struct pmss_ctx_S
{
	sm_magic_T	 sm_magic;
	rcbcom_ctx_T	 pmss_com;
	pmilt_ctx_P	 pmss_pmilt_ctx;	/* pointer back to main ctx */
	int		 pmss_id;	/* SMTPS id */

	unsigned int	 pmss_status;
	uint32_t	 pmss_bit;	/* bit for pmilt_ssctx */

	uint32_t	 pmss_req;
	uint32_t	 pmss_cap;

	unsigned int	 pmss_max_thrs;	/* upper limit for threads */
	unsigned int	 pmss_cur_session;	/* current # of sessions */

	/* head of session list */
	TAILQ_HEAD(, pmse_ctx_S)	 pmss_ss_hd;
};

/* SMTPS status */
#define PMSS_ST_NONE	0x00	/* not yet OK */
#define PMSS_ST_START	0x01	/* started */
#define PMSS_ST_OK	0x10	/* initialized, running normal */
#define PMSS_ST_SH_DOWN	0x80	/* shutting down */
#define PMSS_ST_STOPPED	0x81	/* stopped: almost terminated */


sm_ret_T sm_pmilt_init0(pmilt_ctx_P *_ppmilt_ctx);
sm_ret_T sm_pmilt_init1(pmilt_ctx_P _pmilt_ctx);
sm_ret_T sm_pmilt_start(pmilt_ctx_P _pmilt_ctx);
sm_ret_T sm_pmilt_stop(pmilt_ctx_P _pmilt_ctx);

sm_ret_T sm_pmss_new(pmilt_ctx_P _pmilt_ctx, pmss_ctx_P *_ppmss_ctx);
sm_ret_T sm_pmss_free(pmss_ctx_P _pmss_ctx);

sm_ret_T sm_pmse_new(pmilt_ctx_P _pmilt_ctx, pmss_ctx_P _pmss_ctx, pmse_ctx_P *_ppmse_ctx);
sm_ret_T sm_pmse_free(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx);
sm_ret_T sm_pmse_find(pmss_ctx_P _pmss_ctx, smtp_id_P _se_id, pmse_ctx_P *_ppmse_ctx);

sm_ret_T sm_pmilt_smtpsli(sm_evthr_task_P _tsk);
sm_ret_T sm_smtps2pmilt(sm_evthr_task_P _tsk);

sm_ret_T sm_pmilt_nseid(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, sm_rcbe_P rcbe);
sm_ret_T sm_pmilt_cseid(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx);
sm_ret_T sm_pmilt_helo(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, sm_rcbe_P rcbe);
sm_ret_T sm_pmilt_mail(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, sm_rcbe_P rcbe);
sm_ret_T sm_pmilt_rcpt(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, sm_rcbe_P rcbe);
sm_ret_T sm_pmilt_data(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, sm_rcbe_P rcbe);
sm_ret_T sm_pmilt_body(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, uchar *_buf, size_t _len);
sm_ret_T sm_pmilt_dot(pmss_ctx_P _pmss_ctx, pmse_ctx_P _pmse_ctx, uchar *_buf, size_t _len, sm_rcbe_P rcbe);

#define PMSS_ID_NONE	(-1)	/* no id (yet) */

#define PMILT_R_WAITQ	SM_SUCCESS	/* default: put back in waitq */
#define PMILT_R_ASYNC	1	/* do nothing, task has been put in waitq */

#if PMILTER_DEBUG
#if PMILTER_DEBUG_DEFINE
unsigned int pm_debug;
#else
extern unsigned int pm_debug;
#endif

# define PM_DEBFP	smioerr
# define PM_LEV_DPRINTF(lev, x)	do			\
	{						\
		if ((lev) < pm_debug)			\
			sm_io_fprintf x;		\
	} while (0)

# define PM_DPRINTF(x)	sm_io_fprintf x
#else /* PMILTER_DEBUG */
# define PM_DPRINTF(x)
# define PM_LEV_DPRINTF(lev, x)
#endif /* PMILTER_DEBUG */


#endif /* ! PMILTER_H */
