/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: pmilter_init.c,v 1.4 2005/01/27 00:29:18 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/memops.h"
#include "sm/evthr.h"
#include "pmilter.h"

/*
**  SM_PMILT_INIT0 -- initialize PMILTER, step 0: set default values
**	invoked before command line options are read
**
**	Parameters:
**		ppmilt_ctx -- (pointer to) pmilter context (output)
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmilt_init0(pmilt_ctx_P *ppmilt_ctx)
{
#if 0
	sm_ret_T ret;
#endif
	pmilt_ctx_P pmilt_ctx;

	SM_REQUIRE(ppmilt_ctx != NULL);
	*ppmilt_ctx = NULL;
	pmilt_ctx = (pmilt_ctx_P) sm_zalloc(sizeof(*pmilt_ctx));
	if (pmilt_ctx == NULL)
		return sm_error_temp(SM_EM_PMILTER, ENOMEM);

	/* clear out data */
	sm_memzero(pmilt_ctx, sizeof(*pmilt_ctx));

	/* initialize fds (0 could be valid hence they need to be set) */
	pmilt_ctx->pmilt_sslfd = INVALID_SOCKET;

#if 0
	ret = sm_log_create(NULL, &(pmilt_ctx->pmilt_lctx),
				&(pmilt_ctx->pmilt_lcfg));
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init0, sm_log_create=%x\n", ret);
		goto error;
	}
	ret = sm_log_setfp_fd(pmilt_ctx->pmilt_lctx, smiolog, SMIOLOG_FILENO);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init0, sm_log_setfp=%x\n", ret);
		goto error;
	}
	ret = sm_log_setdebuglevel(pmilt_ctx->pmilt_lctx,
				pmilt_ctx->pmilt_cnf.q_cnf_loglevel);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init0, sm_log_setdebuglevel=%x\n",
			ret);
		goto error;
	}
#endif /* 0 */

	/* this is a bit early ... */
	pmilt_ctx->sm_magic = SM_PMILT_CTX_MAGIC;
	*ppmilt_ctx = pmilt_ctx;
	return SM_SUCCESS;

#if 0
  error:
	return ret;
#endif /* 0 */
}

/*
**  SM_PMILT_INIT1 -- initialize PMILT
**	invoked after command line options are read
**
**	Parameters:
**		pmilt_ctx -- PMILT context
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmilt_init1(pmilt_ctx_P pmilt_ctx)
{
	int i;
	sm_ret_T ret;

	SM_REQUIRE(pmilt_ctx != NULL);

	/* initialize pthreads */
	ret = thr_init();
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init, thr_init=%x\n", ret);
		goto error;
	}

	i = pthread_mutex_init(&(pmilt_ctx->pmilt_mutex), NULL);
	if (i != 0)
	{
		ret = sm_error_perm(SM_EM_Q, i);
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init, mutex_init=%x\n", i);
		goto error;
	}

	/* initialize event threads system */
	/* XXX use config values */
	ret = evthr_init(&(pmilt_ctx->pmilt_ev_ctx), 2, 6, 10);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init, evthr_init=%x\n", ret);
		goto error;
	}

#if 0
	ret = sm_log_setdebuglevel(pmilt_ctx->pmilt_lctx,
				pmilt_ctx->pmilt_cnf.q_cnf_loglevel);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr,
			"sev=ERROR, func=pmilt_init, sm_log_setdebuglevel=%x\n",
			ret);
		goto error;
	}
#endif /* 0 */

	pmilt_ctx->sm_magic = SM_PMILT_CTX_MAGIC;

	/* create pmilter contexts */
	for (i = 0; i < SM_ARRAY_SIZE(pmilt_ctx->pmilt_sctx); i++)
	{
		ret = sm_pmss_new(pmilt_ctx, &(pmilt_ctx->pmilt_sctx[i]));
		if (sm_is_err(ret))
		{
			sm_io_fprintf(smioerr,
				"sev=ERROR, func=pmilt_init, sm_pmss_new=%r, i=%d\n",
				ret, i);
			goto error;
		}
	}


	return SM_SUCCESS;

  error:
	/* XXX need to clean up... */
	return ret;
}
