/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: pmilter_se.c,v 1.7 2005/01/27 00:29:18 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/io.h"
#include "sm/rcb.h"
#include "pmilter.h"

/*
**  SM_PMSE_NEW -- Allocate a new SMTP server context
**
**	Parameters:
**		pmilt_ctx -- PMILTER context
**		pmss_ctx -- PMILTER - SMTP server context
**		ppmse_ctx -- SMTP server context (output)
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmse_new(pmilt_ctx_P pmilt_ctx, pmss_ctx_P pmss_ctx, pmse_ctx_P *ppmse_ctx)
{
	pmse_ctx_P pmse_ctx;

	SM_IS_PMILT_CTX(pmilt_ctx);
	SM_IS_PMSS_CTX(pmss_ctx);
	SM_REQUIRE(ppmse_ctx != NULL);
	pmse_ctx = (pmse_ctx_P) sm_zalloc(sizeof(*pmse_ctx));
	if (pmse_ctx == NULL)
		return sm_error_temp(SM_EM_Q_Q2SS, ENOMEM);
	pmse_ctx->pmse_pmilt_ctx = pmilt_ctx;

	PMSEL_PRE(&(pmss_ctx->pmss_ss_hd), pmse_ctx);
	pmse_ctx->sm_magic = SM_PMSE_CTX_MAGIC;
	*ppmse_ctx = pmse_ctx;
	return SM_SUCCESS;
}

/*
**  SM_PMSE_FREE -- Free a SMTP server context
**
**	Parameters:
**		pmss_ctx -- PMILTER - SMTP server context
**		pmse_ctx -- SMTP server context
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmse_free(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx)
{
	/* XXX this routine assumes nobody accesses pmse_ctx! */
	if (pmse_ctx == NULL)
		return SM_SUCCESS;
	SM_IS_PMSS_CTX(pmss_ctx);

	SM_STR_FREE(pmse_ctx->pmse_arg1);
	SM_STR_FREE(pmse_ctx->pmse_arg2);
	PMSEL_REMOVE(&(pmss_ctx->pmss_ss_hd), pmse_ctx);
	pmse_ctx->sm_magic = SM_MAGIC_NULL;
	sm_free_size(pmse_ctx, sizeof(*pmse_ctx));
	return SM_SUCCESS;
}

/*
**  SM_PMSE_FIND -- Find a SMTP server context based on the session id
**
**	Parameters:
**		pmss_ctx -- PMILTER - SMTP server context
**		se_id -- SMTP session id
**		ppmse_ctx -- SMTP server context (output)
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmse_find(pmss_ctx_P pmss_ctx, smtp_id_P se_id, pmse_ctx_P *ppmse_ctx)
{
	pmse_ctx_P pmse_ctx;

	SM_IS_PMSS_CTX(pmss_ctx);
	SM_REQUIRE(ppmse_ctx != NULL);
	SM_REQUIRE(se_id != NULL);
	SM_REQUIRE(*se_id != '\0');

	if (PMSEL_EMPTY(&(pmss_ctx->pmss_ss_hd)))
		return sm_error_perm(SM_EM_PMILTER, SM_E_NOTFOUND);
	for (pmse_ctx = PMSEL_FIRST(&(pmss_ctx->pmss_ss_hd));
	     pmse_ctx != PMSEL_END(&(pmss_ctx->pmss_ss_hd));
	     pmse_ctx = PMSEL_NEXT(pmse_ctx))
	{
		if (SESSTA_EQ(se_id, pmse_ctx->pmse_se_id))
		{
			*ppmse_ctx = pmse_ctx;
			return SM_SUCCESS;
		}
	}
	return sm_error_perm(SM_EM_PMILTER, SM_E_NOTFOUND);
}
