/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: pmilter_start.c,v 1.6 2005/03/15 19:56:08 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/sysexits.h"
#include "sm/signal.h"
#include "sm/io.h"
#include "sm/unixsock.h"
#include "pmilter.h"

#if SM_USE_PMILTER

/*
**  SM_QM_LISTEN -- start one QMGR listener
**
**	Parameters:
**		sockname -- path for (Unix domain) socket
**			XXX Needs to be fixed for Windows!
**		pfd -- (pointer to) fd (output)
**
**	Returns:
**		usual sm_error code
*/

static sm_ret_T
sm_pmilt_listen(char *sockname, int *pfd)
{
	int lfd, r;
	sm_ret_T ret;

	SM_REQUIRE(sockname != NULL);
	SM_REQUIRE(pfd != NULL);
	ret = SM_SUCCESS;
	r = unlink(sockname);
	if (r < 0 && errno != ENOENT)
	{
		ret = sm_error_perm(SM_EM_Q_START, errno);
		sm_io_fprintf(smioerr, "sev=ERROR, unlink(%s)=%x\n",
			sockname, ret);
		goto error;
	}
	lfd = unix_server_listen(sockname, 10);
	if (!is_valid_socket(lfd))
	{
		ret = sm_error_perm(SM_EM_Q_START, errno);
		sm_io_fprintf(smioerr,
			"sev=ERROR, unix_server_listen(%s)=%x\n", sockname, ret);
		goto error;
	}
	r = chmod(sockname, 0660);
	if (r < 0)
	{
		ret = sm_error_perm(SM_EM_Q_START, errno);
		sm_io_fprintf(smioerr, "sev=ERROR, chmod(%s)=%x\n",
			sockname, ret);
		goto error;
	}
	*pfd = lfd;
	/* FALLTHROUGH */
  error:
	return ret;
}


/*
**  SM_PMILT_STLI -- start PMILTER listeners
**
**	Parameters:
**		pmilt_ctx -- PMILTER context
**
**	Returns:
**		usual sm_error code
**
**	Question: should we store "task" in pmilt_ctx?
*/

static sm_ret_T
sm_pmilt_stli(pmilt_ctx_P pmilt_ctx)
{
	int fd;
	sm_ret_T ret;
	sm_evthr_task_P task;

	SM_IS_PMILT_CTX(pmilt_ctx);

	if (pmilt_ctx->pmilt_sockname == NULL ||
	    *pmilt_ctx->pmilt_sockname == '\0')
		return sm_error_perm(SM_EM_PMILTER, EINVAL);

	/* start listen connections */
	/* these should be in pmilt_ctx after the configuration has been read */

	/* SMTPS */
	ret = sm_pmilt_listen(pmilt_ctx->pmilt_sockname, &fd);
	if (sm_is_err(ret))
		goto error;

	pmilt_ctx->pmilt_sslfd = fd;
	ret = evthr_task_new(pmilt_ctx->pmilt_ev_ctx, &task, EVTHR_EV_LI,
			fd, NULL, sm_pmilt_smtpsli, (void *) pmilt_ctx);
	if (sm_is_err(ret))
		goto error;

#if 0
	pmilt_ctx->pmilt_st_time = evthr_time(pmilt_ctx->pmilt_ev_ctx);
#endif /* 0 */
	return SM_SUCCESS;

  error:
	CLOSE_FD(pmilt_ctx->pmilt_sslfd);
	return ret;
}

/*
**  SM_PMILTER_START -- start PMILTER
**
**	Parameters:
**		pmilt_ctx -- PMILTER context
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_pmilt_start(pmilt_ctx_P pmilt_ctx)
{
	sm_ret_T ret;
#if 0
	sm_evthr_task_P task;
#endif /* 0 */

	SM_IS_PMILT_CTX(pmilt_ctx);

	ret = sm_pmilt_stli(pmilt_ctx);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr, "sev=ERROR, sm_pmilt_stli=%x\n", ret);
		goto error;
	}

#if 0
	ret = evthr_task_new(pmilt_ctx->pmilt_ev_ctx, &task, EVTHR_EV_SG,
			SIGUSR1, NULL, sm_pmilt_usr1, (void *) pmilt_ctx);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr, "sev=ERROR, start_sm_pmilt_usr1=%x\n", ret);
		goto error;
	}

	ret = evthr_task_new(pmilt_ctx->pmilt_ev_ctx, &task, EVTHR_EV_SG,
			SIGUSR2, NULL, sm_pmilt_usr2, (void *) pmilt_ctx);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr, "sev=ERROR, start_sm_pmilt_usr2=%x\n", ret);
		goto error;
	}
#endif /* 0 */

	return SM_SUCCESS;

  error:
	/*
	**  cleanup? tasks may have been started, but PMILTER will terminate
	**  due to the errors.
	*/

	return ret;
}

#endif /* SM_USE_PMILTER */
