/*
 * Copyright (c) 2004 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: pmilter_stop.c,v 1.3 2005/03/15 19:56:08 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/sysexits.h"
#include "sm/signal.h"
#include "sm/io.h"
#include "pmilter.h"

#if SM_USE_PMILTER

/*
**  SM_PMILT_STOP -- stop PMILTER
**
**	Parameters:
**		pmilt_ctx -- PMILTER context
**
**	Returns:
**		usual sm_error code
*/

/* should we pass type of shutdown? immediate, normal, ...? */
sm_ret_T
sm_pmilt_stop(pmilt_ctx_P pmilt_ctx)
{
	int i;
	sm_ret_T ret;

	SM_REQUIRE(pmilt_ctx != NULL);
#if 0
	if (pmilt_ctx->pmilt_status < PMILTER_ST_INIT1)
		return SM_SUCCESS;
#endif /* 0 */

	/*
	**  How to properly shut down the system?
	**  In which order? Which tasks need to be called for termination?
	*/

	CLOSE_FD(pmilt_ctx->pmilt_sslfd);

	if (pmilt_ctx->pmilt_ev_ctx != NULL)
		ret = evthr_stop(pmilt_ctx->pmilt_ev_ctx);
		/* complain on error? */

	for (i = 0; i < SM_ARRAY_SIZE(pmilt_ctx->pmilt_sctx); i++)
	{
		(void) sm_pmss_free(pmilt_ctx->pmilt_sctx[i]);
		pmilt_ctx->pmilt_sctx[i] = NULL;
	}

	/* destroy mutex first? */
	ret = thr_stop();
	if (sm_is_err(ret))
		goto error;

#if 0
	pmilt_ctx->pmilt_status = PMILTER_ST_STOPPED;
#endif /* 0 */

	(void) pthread_mutex_destroy(&(pmilt_ctx->pmilt_mutex));
	return SM_SUCCESS;

  error:
	return ret;
}

#endif /* SM_USE_PMILTER */
