/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-pmilter-0.c,v 1.16 2005/04/30 16:11:16 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/types.h"
#include "sm/sysexits.h"
#include "sm/fcntl.h"
#include "sm/io.h"
#include "sm/ctype.h"
#include "sm/reccom.h"
#include "sm/mta.h"
#include "sm/pmilter.h"
#define PMILTER_DEBUG_DEFINE 1
#include "pmilter.h"
#include "util.h"

typedef struct pmt_ctx_S	pmt_ctx_T, *pmt_ctx_P;

/*
**  pmilter test context
*/

struct pmt_ctx_S
{
	int		 pmt_rcode_nseid;
	int		 pmt_rcode_helo;
	int		 pmt_rcode_mail;
	int		 pmt_rcode_rcpt;
	int		 pmt_rcode_data;
	int		 pmt_rcode_dot;
	uint32_t	 pmt_flags;
	int		 pmt_fd;
	char		 pmt_fname[PATH_MAX];
};

#define PMT_EXIT_CODE	999
#define PMT_EXIT(rcode)	((rcode) == PMT_EXIT_CODE)

#define PMT_FL_NONE	0x0000
#define PMT_FL_W2F	0x0001	/* write to file */
#define PMT_FL_FD_OPEN	0x0002	/* fd is ok to use */
#define PMT_FL_FD_FAIL	0x0004	/* don't try to open file */
#define PMT_FL_WR_FAIL	0x0008	/* write failed before */

#define PMT_SET_FLAG(pmt_ctx, fl)	(pmt_ctx)->pmt_flags |= (fl)
#define PMT_CLR_FLAG(pmt_ctx, fl)	(pmt_ctx)->pmt_flags &= ~(fl)
#define PMT_IS_FLAG(pmt_ctx, fl)	(((pmt_ctx)->pmt_flags & (fl)) != 0)

static pmt_ctx_T pmt_ctx;

sm_ret_T
sm_pmilt_nseid(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, sm_rcbe_P rcbe)
{
	sm_ret_T ret;

	if (PMT_EXIT(pmt_ctx.pmt_rcode_nseid))
		exit(0);
	ret = sm_rcb_putv(&(rcbe->rcbe_rcb), RCB_PUTV_FIRST,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_M2S_ID, pmss_ctx->pmss_id,
		SM_RCBV_BUF, RT_M2S_SEID, pmse_ctx->pmse_se_id, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_M2S_RCODE, pmt_ctx.pmt_rcode_nseid,
		SM_RCBV_END);
	sm_io_fprintf(smioerr, "sev=DBG, nseid=%s, host=%S, ret=%X\n",
		pmse_ctx->pmse_se_id, pmse_ctx->pmse_arg1, ret);
	return ret;
}

sm_ret_T
sm_pmilt_cseid(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx)
{
	return SM_SUCCESS;
}

sm_ret_T
sm_pmilt_helo(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, sm_rcbe_P rcbe)
{
	sm_ret_T ret;

	sm_io_fprintf(smioerr, "sev=DBG, seid=%s, helo=%S\n",
		pmse_ctx->pmse_se_id, pmse_ctx->pmse_arg1);
	if (PMT_EXIT(pmt_ctx.pmt_rcode_helo))
		exit(0);
	ret = sm_rcb_putv(&(rcbe->rcbe_rcb), RCB_PUTV_FIRST,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_M2S_ID, pmss_ctx->pmss_id,
		SM_RCBV_BUF, RT_M2S_SEID, pmse_ctx->pmse_se_id, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_M2S_RCODE, pmt_ctx.pmt_rcode_helo,
		SM_RCBV_END);
	return ret;
}

sm_ret_T
sm_pmilt_mail(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, sm_rcbe_P rcbe)
{
	sm_ret_T ret;
	unsigned int nelem;
	char **argv;

	ret = args2argv(pmse_ctx->pmse_arg2, &nelem, &argv);
	sm_io_fprintf(smioerr,
		"sev=DBG, seid=%s, mail=%@S, args=%S, nelem=%u, args2argv=%r, argv[0]=%s\n",
		pmse_ctx->pmse_se_id, pmse_ctx->pmse_arg1,
		pmse_ctx->pmse_arg2, nelem, ret,
		(argv != NULL && argv[0] != NULL) ? argv[0] : "NULL");
	if (PMT_EXIT(pmt_ctx.pmt_rcode_mail))
		exit(0);
	ret = sm_rcb_putv(&(rcbe->rcbe_rcb), RCB_PUTV_FIRST,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_M2S_ID, pmss_ctx->pmss_id,
		SM_RCBV_BUF, RT_M2S_SEID, pmse_ctx->pmse_se_id, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_M2S_RCODE, pmt_ctx.pmt_rcode_mail,
		SM_RCBV_END);
	if (argv != NULL)
		sm_free(argv);
	return ret;
}

sm_ret_T
sm_pmilt_rcpt(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, sm_rcbe_P rcbe)
{
	sm_ret_T ret;

	sm_io_fprintf(smioerr, "sev=DBG, seid=%s, rcpt=%@S, idx=%d\n",
		pmse_ctx->pmse_se_id, pmse_ctx->pmse_arg1,
		pmse_ctx->pmse_rcpt_idx);
	if (PMT_EXIT(pmt_ctx.pmt_rcode_rcpt))
		exit(0);
	ret = sm_rcb_putv(&(rcbe->rcbe_rcb), RCB_PUTV_FIRST,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_M2S_ID, pmss_ctx->pmss_id,
		SM_RCBV_BUF, RT_M2S_SEID, pmse_ctx->pmse_se_id, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_M2S_RCODE, pmt_ctx.pmt_rcode_rcpt,
		SM_RCBV_END);
	return ret;
}

sm_ret_T
sm_pmilt_data(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, sm_rcbe_P rcbe)
{
	sm_ret_T ret;

	sm_io_fprintf(smioerr, "sev=DBG, seid=%s, where=data\n",
		pmse_ctx->pmse_se_id);
	if (PMT_EXIT(pmt_ctx.pmt_rcode_data))
		exit(0);
	ret = sm_rcb_putv(&(rcbe->rcbe_rcb), RCB_PUTV_FIRST,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_M2S_ID, pmss_ctx->pmss_id,
		SM_RCBV_BUF, RT_M2S_SEID, pmse_ctx->pmse_se_id, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_M2S_RCODE, pmt_ctx.pmt_rcode_data,
		SM_RCBV_END);
	return ret;
}

static void
pm_open(pmse_ctx_P pmse_ctx)
{
	if (PMT_IS_FLAG(&pmt_ctx, PMT_FL_W2F) &&
	    !PMT_IS_FLAG(&pmt_ctx, PMT_FL_FD_OPEN) &&
	    !PMT_IS_FLAG(&pmt_ctx, PMT_FL_FD_FAIL))
	{
		pmt_ctx.pmt_fd = open(pmt_ctx.pmt_fname,
					O_WRONLY|O_APPEND|O_CREAT,
					0660);
		if (pmt_ctx.pmt_fd >= 0)
			PMT_SET_FLAG(&pmt_ctx, PMT_FL_FD_OPEN);
		else
			PMT_SET_FLAG(&pmt_ctx, PMT_FL_FD_FAIL);
		sm_io_fprintf(smioerr,
			"sev=DBG, seid=%s, where=body, open=%d\n",
			pmse_ctx->pmse_se_id, pmt_ctx.pmt_fd);
	}
}

sm_ret_T
sm_pmilt_body(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, uchar *buf, size_t len)
{
	pm_open(pmse_ctx);
	if (PMT_IS_FLAG(&pmt_ctx, PMT_FL_W2F) &&
	    PMT_IS_FLAG(&pmt_ctx, PMT_FL_FD_OPEN) &&
	    !PMT_IS_FLAG(&pmt_ctx, PMT_FL_WR_FAIL) &&
	    buf != NULL && len > 0
	   )
	{
		ssize_t written;

		written = write(pmt_ctx.pmt_fd, buf, len);
		if (written == -1)
		{
			sm_io_fprintf(smioerr,
				"sev=ERROR, seid=%s, where=body, write=%d\n",
				pmse_ctx->pmse_se_id, (int) written);
			PMT_SET_FLAG(&pmt_ctx, PMT_FL_WR_FAIL);
			close(pmt_ctx.pmt_fd);
			pmt_ctx.pmt_fd = INVALID_FD;
			PMT_CLR_FLAG(&pmt_ctx, PMT_FL_FD_OPEN);
		}
	}

	sm_io_fprintf(smioerr, "sev=DBG, seid=%s, where=body, len=%d\n",
		pmse_ctx->pmse_se_id, (int) len);
	return SM_SUCCESS;
}

sm_ret_T
sm_pmilt_dot(pmss_ctx_P pmss_ctx, pmse_ctx_P pmse_ctx, uchar *buf, size_t len, sm_rcbe_P rcbe)
{
	sm_ret_T ret;
	bool writefailed;
	int rcode;

	pm_open(pmse_ctx);
	if (PMT_IS_FLAG(&pmt_ctx, PMT_FL_W2F) &&
	    PMT_IS_FLAG(&pmt_ctx, PMT_FL_FD_OPEN) &&
	    !PMT_IS_FLAG(&pmt_ctx, PMT_FL_WR_FAIL) &&
	    buf != NULL && len > 0
	   )
	{
		ssize_t written;

		written = write(pmt_ctx.pmt_fd, buf, len);
		if (written == -1)
		{
			sm_io_fprintf(smioerr,
				"sev=ERROR, seid=%s, where=body, write=%d\n",
				pmse_ctx->pmse_se_id, (int) written);
			PMT_SET_FLAG(&pmt_ctx, PMT_FL_WR_FAIL);
			close(pmt_ctx.pmt_fd);
			pmt_ctx.pmt_fd = INVALID_FD;
			PMT_CLR_FLAG(&pmt_ctx, PMT_FL_FD_OPEN);
		}
	}

	writefailed = PMT_IS_FLAG(&pmt_ctx, PMT_FL_W2F) &&
			PMT_IS_FLAG(&pmt_ctx, PMT_FL_WR_FAIL);
	if (PMT_IS_FLAG(&pmt_ctx, PMT_FL_W2F) &&
	    PMT_IS_FLAG(&pmt_ctx, PMT_FL_FD_OPEN) &&
	    !PMT_IS_FLAG(&pmt_ctx, PMT_FL_WR_FAIL))
	{
		int r;

		r = close(pmt_ctx.pmt_fd);
		sm_io_fprintf(smioerr,
			"sev=DBG, seid=%s, where=dot, write=%d\n",
			pmse_ctx->pmse_se_id, r);
		pmt_ctx.pmt_fd = INVALID_FD;
		PMT_CLR_FLAG(&pmt_ctx, PMT_FL_FD_OPEN);
	}

	rcode = pmt_ctx.pmt_rcode_dot;
	if (rcode == SMTP_R_OK && writefailed)
		rcode = SMTP_R_TEMP;

	sm_io_fprintf(smioerr, "sev=DBG, seid=%s, where=dot, len=%d\n",
		pmse_ctx->pmse_se_id, (int) len);

	if (PMT_EXIT(pmt_ctx.pmt_rcode_dot))
		exit(0);
	ret = sm_rcb_putv(&(rcbe->rcbe_rcb), RCB_PUTV_FIRST,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_M2S_ID, pmss_ctx->pmss_id,
		SM_RCBV_BUF, RT_M2S_SEID, pmse_ctx->pmse_se_id, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_M2S_RCODE, pmt_ctx.pmt_rcode_dot,
		SM_RCBV_END);
	return ret;
}

/*
**  USAGE -- Print usage message to smioerr
**
**	Parameters:
**		prg -- program name
**
**	Returns:
**		exits
*/

static void
usage(const char *prg)
{
	sm_io_fprintf(smioerr, "usage: %s [options]\n"
		"-f name          write body to file\n"
		"-r stage=rcode   set SMTP reply code for stage to rcode\n"
		"   stage:\n"
		"     c: new session (connect)\n"
		"     h: HELO\n"
		"     m: MAIL\n"
		"     r: RCPT\n"
		"     d: DATA\n"
		"     b: Body\n"
		"   rcode=%d will invoke exit() to simulate a fatal error\n"
		, prg
		, PMT_EXIT_CODE
		);
	exit(EX_USAGE);
}

/*
**  MAIN -- PMILTER test server
**
**	Parameters:
**		argc -- number of arguments
**		argv -- vector of arguments
**
**	Returns:
**		exit code
*/

int
main(int argc, char *argv[])
{
	sm_ret_T ret;
	int c;
	unsigned int u, off;
	pmilt_ctx_P pmilt_ctx;
	char *prog;

	prog = argv[0];
	pmilt_ctx = NULL;
	if (getuid() == 0 || geteuid() == 0)
	{
		sm_io_fprintf(smioerr,
			"%s: ERROR: do not run this as super-user!",
			prog);
		exit(EX_USAGE);
	}

	/* initialize test context */
	sm_memzero(&pmt_ctx, sizeof(pmt_ctx));
	pmt_ctx.pmt_rcode_nseid = SMTP_R_OK;
	pmt_ctx.pmt_rcode_helo = SMTP_R_OK;
	pmt_ctx.pmt_rcode_mail = SMTP_R_OK;
	pmt_ctx.pmt_rcode_rcpt = SMTP_R_OK;
	pmt_ctx.pmt_rcode_data = SMTP_R_OK;
	pmt_ctx.pmt_rcode_dot = SMTP_R_OK;
	pmt_ctx.pmt_fname[0] = '\0';
	pmt_ctx.pmt_fd = INVALID_FD;

	/* basic initialization */
	ret = sm_pmilt_init0(&pmilt_ctx);
	if (sm_is_err(ret))
		goto error0;
#if SM_HEAP_CHECK
	SmHeapCheck = 1;
	if (HEAP_CHECK)
		sm_heap_report(smioerr, 3);
#endif /* SM_HEAP_CHECK */

	while ((c = getopt(argc, argv, "f:r:")) != -1)
	{
		switch (c)
		{
		  case 'f':
			strlcpy(pmt_ctx.pmt_fname, optarg,
				sizeof(pmt_ctx.pmt_fname));
			PMT_SET_FLAG(&pmt_ctx, PMT_FL_W2F);
			break;
		  case 'r':
			if (optarg != NULL && ISALPHA(optarg[0])
			    && optarg[1] == '=')
				off = 2;
			else
			{
				usage(prog);
				break;
			}

			u = (unsigned int) atoi(optarg + off);
			switch (optarg[0])
			{
			  case 'c':
				pmt_ctx.pmt_rcode_nseid = u;
				break;

			  case 'h':
			  case 'e':
				pmt_ctx.pmt_rcode_helo = u;
				break;

			  case 'm':
				pmt_ctx.pmt_rcode_mail = u;
				break;

			  case 'r':
				pmt_ctx.pmt_rcode_rcpt = u;
				break;

			  case 'd':
				pmt_ctx.pmt_rcode_data = u;
				break;

			  case 'b':
				pmt_ctx.pmt_rcode_dot = u;
				break;
			  default:
				usage(prog);
				break;
			}
			break;
		  default:
			usage(prog);
			break;
		}
	}

	argc -= optind;
	argv += optind;

	/* XXX HACK */
	if (argc > 0)
		pmilt_ctx->pmilt_sockname = argv[0];
	else
		pmilt_ctx->pmilt_sockname = "pmilter.sock";

#if 0
	/* read config */
	ret = sm_pmilt_rdcf(pmilt_ctx, argc, argv);
	if (sm_is_err(ret))
		goto error0;
#endif /* 0 */

	/* initialize system */
	ret = sm_pmilt_init1(pmilt_ctx);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr, "sev=ERROR, sm_pmilt_init=%x\n", ret);
		goto error;
	}

	/* startup pmilt */
	ret = sm_pmilt_start(pmilt_ctx);
	if (sm_is_err(ret))
	{
		sm_io_fprintf(smioerr, "sev=ERROR, sm_pmilt_start=%x\n", ret);
		goto error;
	}

	/* wait for events, schedule tasks */
	ret = evthr_loop(pmilt_ctx->pmilt_ev_ctx);

	ret = sm_pmilt_stop(pmilt_ctx);
#if SM_HEAP_CHECK
	if (HEAP_CHECK)
		sm_heap_report(smioerr, 3);
#endif /* SM_HEAP_CHECK */
	return 0;

  error:
	/* ignore shutdown errors... */
	(void) sm_pmilt_stop(pmilt_ctx);

  error0:
	/* select an appropriate error here... */
	return sm_error_value(ret);
}
