#!/bin/sh
# $Id: t-access-1.sh,v 1.4 2005/03/08 00:30:01 ca Exp $
# Copyright (c) 2004 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ------------------------------------------------------------
# Description:
# Script to test access map restrictions for RFC2821 addresses.
# Uses smar, t-access-0, access map, t-mm, startsmar.sh
# ------------------------------------------------------------
# Note: smar and t-access-0 are running asynchronous, hence
# there must be no expected order in the output file.
#
# don't run if stop is "set".
test -s stop && exit 1
#

# path to programs
P=..
SD=`dirname $0`
SHELL=/bin/sh
OUT=access-1.out
IP=access-1.ip
OK=access-1.ok
ACC=access
ADB=${ACC}.db
SOCK=qmsmar/qmsmar
PIDS=pids
ERRS=0
TPRG=./t-access-0

rm -f ${ADB} ${SOCK} ${PIDS}

# create access map for tests
${VALGRIND} ../libsmmap/t-mm -t'	' -F ${ADB} <<EOF
to:not	error:550 5.7.0 no to:not
to:not1@b.c	error:551 5.7.1 no to:not1@b.c
to:@not2.c	error:552 5.7.2 no to:@not2.c
EOF

killit()
{
if test -s ${PIDS}
then
  # stop SMAR
  for i in `cat ${PIDS}`
  do
    kill ${i}
  done
fi
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
exit 1
}
trap killit 2


# start SMAR
${SD}/startsmar.sh || exit 1
rm -f ${OUT} ${IP} ${OK}

# test
${VALGRIND} ${TPRG} -r '<not@test4.dom>' > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='550 5.7.0 no to:not
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && exit ${ERRS}
fi

# test
${VALGRIND} ${TPRG} -r '<not1@b.c>' '<any2@not2.c>' > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='551 5.7.1 no to:not1@b.c
react=statt, ret=0, statt='552 5.7.2 no to:@not2.c
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && exit ${ERRS}
fi

# test
${VALGRIND} ${TPRG} -r '<not1@b.c.d>' '<any2@not2.d>' > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=no_match
react=no_match
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  test X"${STOPONERROR}" != X && exit ${ERRS}
fi

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
sleep 3

exit ${ERRS}
