#!/bin/sh
# $Id: t-smar-3.sh,v 1.14 2005/03/08 00:30:01 ca Exp $
# Copyright (c) 2003, 2004 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ------------------------------------------------------------
# Description:
# Script to test access map restrictions (only IP addresses)
# and reverse lookups (PTR).
# Uses smar, t-smar-3, access map, t-mm, startsmar.sh
# ------------------------------------------------------------
#
test -s stop && exit 1
#

if test X"${SM_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0"
  exit 0
fi

# path to programs
P=..
SD=`dirname $0`
SHELL=/bin/sh
OUT=smar-3.out
IP=smar-3.ip
OK=smar-3.ok
ACC=access
ADB=${ACC}.db
SOCK=qmsmar/qmsmar
PIDS=pids
ERRS=0
TEST=0
VERBOSE=false
TPRG=./t-smar-3

if test X"`uname`" != "XOSF1"
then
while getopts V FLAG
do
  case "${FLAG}" in
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# some IP addresses which don't have reverse mapping....
NORVRSIP=${SM_NORVRSIP:-10.255.255.255}
NORVRSIPOK=${SM_NORVRSIP:-10.255.255.0}

rm -f ${ADB} ${SOCK} ${PIDS}

# create access map for tests
../libsmmap/t-mm -t'	' -F ${ADB} <<EOF
cltaddr:1.2.3.4	error:550 5.7.1 access denied
cltaddr:10.12.13.14	quick:error:551 5.7.2 access quickly denied
cltaddr:${NORVRSIPOK}	OK
cltaddr:134.245.248.42	error:450 4.7.1 hey
cltresolve:NO	error:450 4.7.1 get a name
EOF

# start SMAR
${SD}/startsmar.sh || exit 1
rm -f ${OUT} ${IP} ${OK}

# ----------------------------------------
# test: PTR lookup of services.sendmail.org
# Note: increase timeout?
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} '209.246.26.22' > ${OUT} 2>&1

grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rvrs_st, ret=0, stat=1
react=rvrs_name, ret=0, name=services.sendmail.org.
EOF

if diff -i ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# test: mine.informatik.uni-kiel.de.
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} '134.245.248.42' > ${OUT} 2>&1

grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='450 4.7.1 hey
react=rvrs_st, ret=0, stat=1
react=rvrs_name, ret=0, name=mine.informatik.uni-kiel.de.
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# test: cltaddr
# THIS DOESN'T WORK reliably! someone could have an entry in his local DNS.
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} '1.2.3.4' > ${OUT} 2>&1

grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='550 5.7.1 access denied
react=rvrs_st, ret=0, stat=2
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# test: quick
# THIS DOESN'T WORK reliably! someone could have an entry in his local DNS.
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} '10.12.13.14' > ${OUT} 2>&1

grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='551 5.7.2 access quickly denied
react=rvrs_st, ret=0, stat=2
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi


# ----------------------------------------
# THIS DOESN'T WORK reliably! someone could have an entry in his local DNS.
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} ${NORVRSIP} > ${OUT} 2>&1

grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='450 4.7.1 get a name
react=rvrs_st, ret=0, stat=2
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# THIS DOESN'T WORK reliably! someone could have an entry in his local DNS.
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} ${NORVRSIPOK} > ${OUT} 2>&1

grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rvrs_st, ret=0, stat=2
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# How to test for the various errors?
# temporary lookup failures
# permanent lookup failures
# forward != revervse lookup

# run the tests only if ${SM_HAVE_10_INADDRARPA} is set
# this requires DNS entries for 10.255.255.*
if test X"${SM_HAVE_10_INADDRARPA}" = "Xtrue"
then

# ----------------------------------------
# match
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} 10.255.255.1 > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rvrs_st, ret=0, stat=1
react=rvrs_name, ret=0, name=one.sm9.org.
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# does not match
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} 10.255.255.2 > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=statt, ret=0, statt='450 4.7.1 get a name
react=rvrs_st, ret=0, stat=2
react=rvrs_name, ret=0, name=two.sm9.org.
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# forward lookup fails
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} 10.255.255.3 > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rvrs_st, ret=0, stat=8
react=rvrs_name, ret=0, name=three.sm9.org.
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# ----------------------------------------
# reverse lookup fails
TEST=`expr ${TEST} + 1 `
${VALGRIND} ${TPRG} 10.255.255.10 > ${OUT} 2>&1
grep '^react=' ${OUT} > ${IP}
cat > ${OK} <<EOF
react=rvrs_st, ret=0, stat=4
EOF
if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo ${TEST} failed
fi

# test X"${SM_HAVE_10_INADDRARPA}" = "Xtrue"
fi

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
sleep 3

exit ${ERRS}
