/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

/*
**  SMTPS - policy milter communication module
*/

#include "sm/generic.h"
SM_RCSID("@(#)$Id: s2m.c,v 1.12 2005/03/15 19:56:13 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/reccom.h"
#include "statethreads/st.h"
#include "sm/str.h"
#include "sm/rcbst.h"
#include "sm/stthreads.h"
#include "smtps-str.h"
#include "s2q.h"
#include "smtps.h"
#include "s2m.h"

#if SM_USE_PMILTER

/*
**  SM_S2M_CLT -- send client address to PMILTER
**
**	Parameters:
**		ss_sess -- session context
**		s2m_ctx -- S2M context
**		smtps_id -- id of SMTPS
**		sid -- session id
**		cltipv4 -- client IPv4 address
**		port -- port of server
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_s2m_clt(ss_sess_P ss_sess, s2q_ctx_P s2m_ctx, int smtps_id, sessta_id_P sid, uint32_t cltipv4, ushort port)
{
	sm_ret_T ret;
	sm_rcb_P rcb;

	SM_REQUIRE(s2m_ctx != NULL);
	SM_IS_SS_SESS(ss_sess);
	if (S2Q_IS_IOERR(s2m_ctx))
		return sm_error_temp(SM_EM_SMTPS, EIO);

	rcb = ss_sess->ssse_rcb;
	ret = sm_rcb_putrec(rcb, 0, S2Q_RCB_SIZE,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_S2M_ID, smtps_id,
		SM_RCBV_BUF, RT_S2M_NSEID, sid, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_S2M_IPV4, cltipv4,
		SM_RCBV_INT, RT_S2M_PORT, port,
		SM_RCBV_CSTR, RT_S2M_HOST, ss_sess->ssse_cltname,
		SM_RCBV_END);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_clt, sess=%p, sm_rcb_putrec=%r\n",
			ss_sess, ret));
		goto error;
	}
	ret = ss_send_rq(ss_sess, sid, s2m_ctx, rcb, true);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_clt, sess=%p, ss_send_rq=%r\n",
			ss_sess, ret));
		goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}

/*
**  SM_S2M_HELO -- send HELO string to PMILTER
**
**	Parameters:
**		ss_sess -- session context
**		s2m_ctx -- S2M context
**		smtps_id -- id of SMTPS
**		sid -- session id
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_s2m_helo(ss_sess_P ss_sess, s2q_ctx_P s2m_ctx, int smtps_id, sessta_id_P sid)
{
	sm_ret_T ret;
	sm_rcb_P rcb;

	SM_REQUIRE(s2m_ctx != NULL);
	SM_IS_SS_SESS(ss_sess);
	if (S2Q_IS_IOERR(s2m_ctx))
		return sm_error_temp(SM_EM_SMTPS, EIO);

	rcb = ss_sess->ssse_rcb;
	ret = sm_rcb_putrec(rcb, 0, S2Q_RCB_SIZE,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_S2M_ID, smtps_id,
		SM_RCBV_BUF, RT_S2M_SEID, sid, SMTP_STID_SIZE,
		SM_RCBV_STR, RT_S2M_HELO, ss_sess->ssse_helo,
		SM_RCBV_END);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_helo, sess=%p, sm_rcb_putrec=%r\n",
			ss_sess, ret));
		goto error;
	}
	ret = ss_send_rq(ss_sess, sid, s2m_ctx, rcb, true);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_addr, sess=%p, ss_send_rq=%r\n",
			ss_sess, ret));
		goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}

/*
**  SM_S2M_MAIL -- send mail address to PMILTER
**
**	Parameters:
**		ss_sess -- session context
**		s2m_ctx -- S2M context
**		smtps_id -- id of SMTPS
**		sid -- session id
**		tid -- transaction id
**		mail -- mail from address
**		argoffset -- offset of arguments in sess_rd (0: no args)
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_s2m_mail(ss_sess_P ss_sess, s2q_ctx_P s2m_ctx, int smtps_id, sessta_id_P sid, sessta_id_P tid, sm_str_P mail, uint argoffset)
{
	sm_ret_T ret;
	sm_rcb_P rcb;

	SM_REQUIRE(s2m_ctx != NULL);
	SM_IS_SS_SESS(ss_sess);
	if (S2Q_IS_IOERR(s2m_ctx))
		return sm_error_temp(SM_EM_SMTPS, EIO);
	if (argoffset > 0)
		SM_REQUIRE(sm_str_getlen(ss_sess->ssse_rd) > argoffset);

	rcb = ss_sess->ssse_rcb;
	ret = sm_rcb_putrec(rcb, 0, S2Q_RCB_SIZE,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_S2M_ID, smtps_id,
		SM_RCBV_BUF, RT_S2M_SEID, sid, SMTP_STID_SIZE,
		SM_RCBV_BUF, RT_S2M_NTAID, tid, SMTP_STID_SIZE,
		SM_RCBV_STR, RT_S2M_MAIL, mail,
		SM_RCBV_BUF, (argoffset > 0) ? RT_S2M_ARG : RT_NOSEND,
			sm_str_getdata(ss_sess->ssse_rd) + argoffset,
			sm_str_getlen(ss_sess->ssse_rd) - argoffset,
		SM_RCBV_END);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_mail, sess=%p, sm_rcb_putrec=%r\n",
			ss_sess, ret));
		goto error;
	}
	ret = ss_send_rq(ss_sess, sid, s2m_ctx, rcb, true);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_mail, sess=%p, ss_send_rq=%r\n",
			ss_sess, ret));
		goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}

/*
**  SM_S2M_RCPT -- send recipient to PMILTER
**
**	Parameters:
**		ss_sess -- session context
**		s2m_ctx -- S2M context
**		smtps_id -- id of SMTPS
**		sid -- session id
**		rcpt_idx -- rcpt idx
**		rcpt -- rcpt to address
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_s2m_rcpt(ss_sess_P ss_sess, s2q_ctx_P s2m_ctx, int smtps_id, sessta_id_P sid, int rcpt_idx, sm_str_P rcpt)
{
	sm_ret_T ret;
	sm_rcb_P rcb;

	SM_REQUIRE(s2m_ctx != NULL);
	SM_IS_SS_SESS(ss_sess);
	if (S2Q_IS_IOERR(s2m_ctx))
		return sm_error_temp(SM_EM_SMTPS, EIO);

	rcb = ss_sess->ssse_rcb;
	ret = sm_rcb_putrec(rcb, 0, S2Q_RCB_SIZE,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_S2M_ID, (uint32_t) smtps_id,
		SM_RCBV_BUF, RT_S2M_SEID, sid, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_S2M_RCPT_IDX, (uint32_t) rcpt_idx,
		SM_RCBV_STR, RT_S2M_RCPT, rcpt,
		SM_RCBV_END);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_rcpt, sess=%p, sm_rcb_putrec=%r\n",
			ss_sess, ret));
		goto error;
	}
	ret = ss_send_rq(ss_sess, sid, s2m_ctx, rcb, true);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_rcpt, sess=%p, ss_send_rq=%r\n",
			ss_sess, ret));
		goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}

/*
**  SM_S2M_DATA -- inform PMILTER about DATA command
**
**	Parameters:
**		ss_sess -- session context
**		s2m_ctx -- S2M context
**		smtps_id -- id of SMTPS
**		sid -- session id
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_s2m_data(ss_sess_P ss_sess, s2q_ctx_P s2m_ctx, int smtps_id, sessta_id_P sid)
{
	sm_ret_T ret;
	sm_rcb_P rcb;

	SM_REQUIRE(s2m_ctx != NULL);
	SM_IS_SS_SESS(ss_sess);
	if (S2Q_IS_IOERR(s2m_ctx))
		return sm_error_temp(SM_EM_SMTPS, EIO);

	rcb = ss_sess->ssse_rcb;
	ret = sm_rcb_putrec(rcb, 0, S2Q_RCB_SIZE,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_S2M_ID, (uint32_t) smtps_id,
		SM_RCBV_BUF, RT_S2M_SEID, sid, SMTP_STID_SIZE,
		SM_RCBV_INT, RT_S2M_DATA, 0,
		SM_RCBV_END);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_data, sess=%p, sm_rcb_putrec=%r\n",
			ss_sess, ret));
		goto error;
	}
	ret = ss_send_rq(ss_sess, sid, s2m_ctx, rcb, true);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_data, sess=%p, ss_send_rq=%r\n",
			ss_sess, ret));
		goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}

/*
**  SM_S2M_BODY -- send BODY chunk to PMILTER
**
**	Parameters:
**		ss_sess -- session context
**		s2m_ctx -- S2M context
**		smtps_id -- id of SMTPS
**		sid -- session id
**		buf -- body chunk
**		len -- length of body chunk
**		last -- last body chunk?
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_s2m_body(ss_sess_P ss_sess, s2q_ctx_P s2m_ctx, int smtps_id, sessta_id_P sid, uchar *buf, size_t len, bool last)
{
	sm_ret_T ret;
	sm_rcb_P rcb;

	SM_REQUIRE(s2m_ctx != NULL);
	SM_IS_SS_SESS(ss_sess);
	if (S2Q_IS_IOERR(s2m_ctx))
		return sm_error_temp(SM_EM_SMTPS, EIO);
	if (len == 0)
		return SM_SUCCESS;
	SM_REQUIRE(buf != NULL);

	rcb = ss_sess->ssse_rcb;
	ret = sm_rcb_putrec(rcb, 0, -1,
		SM_RCBV_INT, RT_PROT_VER, PROT_VER_RT,
		SM_RCBV_INT, RT_S2M_ID, (uint32_t) smtps_id,
		SM_RCBV_BUF, RT_S2M_SEID, sid, SMTP_STID_SIZE,
		SM_RCBV_BUF, last ? RT_S2M_DOT : RT_S2M_BODY, buf, len,
		SM_RCBV_END);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_body, sess=%p, sm_rcb_putrec=%r\n",
			ss_sess, ret));
		goto error;
	}
	ret = ss_send_rq(ss_sess, sid, s2m_ctx, rcb, true);
	if (sm_is_err(ret))
	{
		SSQ_DPRINTF((smioerr,
			"sev=ERROR, func=sm_s2m_body, sess=%p, ss_send_rq=%r\n",
			ss_sess, ret));
		goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}
#endif /* SM_USE_PMILTER */
